@@ -143,7 +143,7 @@ Usage:
     returning: 4
     results: (0, 1, 2, 3, 4)
 
-As is expected, all CPU cores were used and we were hardware-efficient!
+As expected, all CPU cores were used and we were hardware-efficient!
 
 ## Solving IO bound tasks efficiently
 
@@ -169,7 +169,7 @@ Usage:
             results = await collect([
                 # in_thread sends the task to a pool of threads
                 in_thread(io_bound_task, id)
-                # Let's solve 5 of those tasks in parallel!
+                # Let's solve 5 of those tasks concurrently!
                 for id in range(5)
             ])
             print('time:', time(), 'results:', results)
@@ -187,9 +187,9 @@ Usage:
     time: 1597623832 returning: 1
     time: 1597623832 results: (0, 1, 2, 3, 4)
 
-As is expected, all tasks were executed concurrently. This means that
-instead of waiting five seconds for five tasks (in serial) we just waited one
-second for all of them.
+As expected, all tasks were executed concurrently. This means that instead of
+waiting five seconds for five tasks (serially) we just waited one second for
+all of them.
 
 # Installing
 
@@ -198,14 +198,6 @@ second for all of them.
 # Using
 
     >>> from aioextensions import *  # to import everything
-    >>> from aioextensions import (  # recommended way
-            coolect,
-            in_thread,
-            in_process,
-            resolve,
-            schedule,
-            # ...
-        )
 
 Please read the documentation bellow for more details about every function.
 """
@@ -217,6 +209,7 @@ from concurrent.futures import (
     ProcessPoolExecutor,
     ThreadPoolExecutor,
 )
+from contextlib import suppress
 from functools import (
     partial,
     wraps,
@@ -229,10 +222,12 @@ from os import (
 )
 from typing import (
     Any,
+    AsyncGenerator,
     Awaitable,
     Callable,
     cast,
     Dict,
+    Generator,
     Iterable,
     Optional,
     Tuple,
@@ -245,9 +240,10 @@ from typing import (
 import uvloop
 
 # Constants
-CPU_COUNT: int = cpu_count() or 1
 F = TypeVar('F', bound=Callable[..., Any])  # pylint: disable=invalid-name
+S = TypeVar('S')  # pylint: disable=invalid-name
 T = TypeVar('T')  # pylint: disable=invalid-name
+Y = TypeVar('Y')  # pylint: disable=invalid-name
 
 # Linters
 # pylint: disable=unsubscriptable-object
@@ -257,6 +253,7 @@ def run(coroutine: Awaitable[T], *, debug: bool = False) -> T:
     """Execute an asynchronous function synchronously and return its result.
 
     Example:
+
         >>> async def do(a, b=0):
                 await something
                 return a + b
@@ -291,13 +288,12 @@ async def in_thread(
     memory footprint.
 
     .. warning::
-        Executing CPU intensive work here is a bad idea because of the limitations
-        that the [GIL](https://realpython.com/python-gil) imposes.
+        Executing CPU intensive work here is a bad idea because of the
+        limitations that the [GIL](https://realpython.com/python-gil) imposes.
 
         See `in_process` for a CPU performant alternative.
     """
-    if not THREAD_POOL.initialized:
-        THREAD_POOL.initialize(max_workers=10 * CPU_COUNT)
+    _ensure_thread_pool_is_initialized()
 
     return await asyncio.get_running_loop().run_in_executor(
         THREAD_POOL.pool, partial(function, *args, **kwargs),
@@ -327,8 +323,7 @@ async def in_process(
 
         See `in_thread` for an IO performant alternative.
     """
-    if not PROCESS_POOL.initialized:
-        PROCESS_POOL.initialize(max_workers=CPU_COUNT)
+    _ensure_process_pool_is_initialized()
 
     return await asyncio.get_running_loop().run_in_executor(
         PROCESS_POOL.pool, partial(function, *args, **kwargs),
@@ -368,7 +363,7 @@ async def collect(
     Usage:
         >>> async def do(n):
                 print('running:', n)
-                await asyncio.sleep(1)
+                await sleep(1)
                 print('returning:', n)
                 return n
 
@@ -534,31 +529,117 @@ async def force_loop_cycle() -> None:
     await asyncio.sleep(0)
 
 
+async def generate_in_thread(
+    generator_func: Callable[..., Generator[Y, S, None]],
+    *args: Any,
+    **kwargs: Any,
+) -> AsyncGenerator[Y, S]:
+    """Mimic `generator_func(*args, **kwargs)` in the configured thread pool.
+
+    Note that `generator_func(*args, **kwargs)` may return a generator or an
+    interator and both cases are handled rightfully.
+
+    Usage:
+
+        >>> from os import scandir
+
+        >>> async for entry in generate_in_thread(scandir, '.'):
+                print(entry.name)
+
+    Output:
+        ```
+        .gitignore
+        LICENSE.md
+        README.md
+        ...
+        ```
+
+    Calls to the generator are done serially and not concurrently.
+
+    The benefit of wrapping a generator with this function is that the
+    event-loop is free to schedule and wait another tasks in the mean time.
+    For instance, in a web server.
+    """
+    gen: Generator[Y, S, None] = generator_func(*args, **kwargs)
+    gen_sent: Any = None
+
+    def gen_next(val: S) -> Y:
+        with suppress(StopIteration):
+            return gen.send(val) if hasattr(gen, 'send') else next(gen)
+        raise StopAsyncIteration()
+
+    while True:
+        try:
+            gen_sent = yield await in_thread(gen_next, gen_sent)
+        except StopAsyncIteration:
+            return
+
+
 def schedule(
     awaitable: Awaitable[T],
     *,
     loop: Optional[asyncio.AbstractEventLoop] = None,
-) -> Awaitable[T]:
+) -> 'Awaitable[asyncio.Future[T]]':
     """Schedule an awaitable in the event loop and return a wrapper for it.
+
+    Usage:
+
+        >>> async def do(n):
+                print('running:', n)
+                await sleep(1)
+                print('returning:', n)
+
+        >>> task = schedule(do(3))  # Task is executing in the background now
+
+        >>> print('other work is being done here')
+
+        >>> task_result = await task  # Wait until the task is ready
+
+        >>> print(task_result.result())  # may rise if do() raised
+
+    Output:
+
+        ```
+        other work is being done here
+        doing: 3
+        returning: 3
+        3
+        ```
+
+    This works very similar to asyncio.create_task. The main difference is that
+    the result (or exception) can be accessed via exception() or result()
+    methods.
+
+    If an exception was raised by the awaitable, it will be propagated only at
+    the moment result() is called and never otherwise.
     """
     wrapper = (loop or asyncio.get_event_loop()).create_future()
 
     def _done_callback(future: asyncio.Future) -> None:
         if not wrapper.done():
-            if future.cancelled():
-                wrapper.cancel()
-            elif future.exception():
-                wrapper.set_exception(future)
-            else:
-                wrapper.set_result(future)
+            wrapper.set_result(future)
 
     asyncio.create_task(awaitable).add_done_callback(_done_callback)
 
     return wrapper
 
 
-class ExecutorPool:
+def _ensure_process_pool_is_initialized() -> None:
+    if not PROCESS_POOL.initialized:
+        PROCESS_POOL.initialize(max_workers=CPU_CORES)
 
+
+def _ensure_thread_pool_is_initialized() -> None:
+    if not THREAD_POOL.initialized:
+        THREAD_POOL.initialize(max_workers=10 * CPU_CORES)
+
+
+class ExecutorPool:
+    """Object representing a pool of Processes or Threads.
+
+    The actual pool is created at `initialization` time
+    and it is empty until that.
+    """
     def __init__(
         self,
         cls: Union[
@@ -570,18 +651,26 @@ class ExecutorPool:
         self._pool: Optional[Executor] = None
 
     def initialize(self, *, max_workers: Optional[int] = None) -> None:
+        """Initialize the executor with a cap of at most `max_workers`.
+
+        Workers are created on-demand as needed or never created at all
+        if never needed.
+        """
         if self._pool is not None:
             self._pool.shutdown(wait=False)
 
         self._pool = self._cls(max_workers=max_workers)
 
     def shutdown(self, *, wait: bool) -> None:
+        """Shut down the executor and (optionally) waits for workers to finish.
+        """
         if self._pool is not None:
             self._pool.shutdown(wait=wait)
             self._pool = None
 
     @property
     def pool(self) -> Executor:
+        """Low level pool of workers held by the executor, may be None."""
         if self._pool is None:
             raise RuntimeError('Must call initialize first')
 
@@ -589,6 +678,7 @@ class ExecutorPool:
 
     @property
     def initialized(self) -> bool:
+        """Return true if the executor is initialized and ready to process."""
         return self._pool is not None
 
 
@@ -614,5 +704,23 @@ def run_decorator(function: F) -> F:
 
 
 # Constants
+CPU_CORES: int = cpu_count() or 1
+"""Number of CPU cores in the host system."""
+
 PROCESS_POOL: ExecutorPool = ExecutorPool(ProcessPoolExecutor)
+"""Process pool used by `in_process` function to execute work.
+
+Preconfigured to launch at most `CPU_CORES` processes (if needed).
+
+Proceses are created on the first `in_process` call, one by one as needed
+or never launched otherwise.
+"""
+
 THREAD_POOL: ExecutorPool = ExecutorPool(ThreadPoolExecutor)
+"""Thread pool used by `in_thread` function to execute work.
+
+Preconfigured to launch at most 10 * `CPU_CORES` threads (if needed).
+
+Threads are created on the first `in_thread` call, one by one as needed,
+or never launched otherwise.
+"""
